#include "common.h"
#include "com.h"
#include "kcomm.h"
#include "mainWnd.h"
#include "addrWnd.h"

HWND hAddrWnd = NULL;

void getAddresses(HWND hWnd)
{
	BYTE cmd[6];
	HANDLE hcom;
	BYTE* data;
	DWORD size;

	Log("<<GetAddress>>\r\n");

	Log("beginComm...");
	if(beginComm(&hcom))
	{
		Log("failed\r\n");
		return;
	}
	Log("OK\r\n");

	Log("Getting AddressList...");
	cmd[0] = 0x02;
	cmd[1] = 0x20;
	cmd[2] = 0x01;
	cmd[3] = 0x03;
	cmd[4] = 0x00;
	cmd[5] = 0x00;
	if(sendCommand(hcom, cmd, 6, FALSE))
	{
		Log("Send failed\r\n");
		goto error;
	}
	if(ReadByte(hcom) != 0x06)
	{
		Log("NAK\r\n");
		goto error;
	}
	if(getResult(hcom, &data, &size))
	{
		Log("Read error\r\n");
		goto error;
	}
	Log("OK\r\n");

	Log("endComm...");
	if(endComm(hcom))
	{
		Log("failed\r\n");
		return;
	}
	Log("OK");

	Log("Parsing...");
	{
		BYTE* b = data;
		DWORD i = 0;

		ListView_DeleteAllItems(GetDlgItem(hWnd, IDC_ADDRLIST));

		while((DWORD)(b - data) < size)
		{
			BOOL inserted = FALSE;

			if(*b != 0x1C)
			{
				Log("RecordFirst not found\r\n");
				goto parseError;
			}
			b++;

			while((DWORD)(b - data) < size)
			{
				WORD psize;
				char buf[256];
				BYTE cmd;
				int telcount = 0;
				int teltypecount = 0;
				int emlcount = 0;
				int emltypecount = 0;

				if(*b == 0x1C)
					break;

				if(*b != 0x1E)
				{
					Log("RecordByte not found\r\n");
					goto parseError;
				}
				b++;

				cmd = *b++;

				psize = (b[0] << 8) | b[1];
				b += 2;

				if(!inserted && cmd != 0x0B)
				{
					LV_ITEM lvi;

					lvi.mask = 0;
					lvi.iItem = i;
					lvi.iSubItem = 0;
					ListView_InsertItem(GetDlgItem(hWnd, IDC_ADDRLIST), &lvi);

					inserted = TRUE;
				}

				switch(cmd)
				{
				case 0x01:	// name
					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 0, buf);
					break;
				case 0x02:	// furigana
					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 1, buf);
					break;
				case 0x03:	// tel
					if(telcount)
						break;

					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 2, buf);

					telcount++;
					break;
				case 0x04:	// tel type
					if(teltypecount)
						break;

					{
						char* type[] = {"00", "Work", "Home", "FAX", "CellPhone", "PHS", "-"};
						if(b[0] <= 0x06)
							strcopy(buf, type[b[0]]);
						else
							wsprintf(buf, "%02X", b[0]);
						ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 3, buf);
					}

					teltypecount++;
					break;
				case 0x05:	// email
					if(telcount)
						break;

					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 4, buf);

					emlcount++;
					break;
				case 0x06:	// email type
					if(teltypecount)
						break;

					{
						char* type[] = {"00", "01", "Home", "CellPhone", "PHS", "School", "Work", "-"};
						if(b[0] <= 0x07)
							strcopy(buf, type[b[0]]);
						else
							wsprintf(buf, "%02X", b[0]);
						ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 5, buf);
					}

					emltypecount++;
					break;
				case 0x0C:	// Memo
					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 6, buf);
					break;
				case 0x0A:	// Group
					wsprintf(buf, "%d", (b[0] << 8) | b[1]);
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 7, buf);
					break;
				case 0x0B:	// Mem No.
					wsprintf(buf, "%d", (b[0] << 8) | b[1]);
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 8, buf);
					break;
				case 0x08:	// Addr
					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 9, buf);
					break;
				case 0x0E:	// GPS
					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 10, buf);
					break;
				case 0x0F:	// Picture
					strncopy(buf, b, strlength(b));
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 11, buf);
					break;
				case 0x10:	// Reg.data
					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 12, buf);
					break;
				case 0x11:	// blood type
					{
						char* btype[] = {"-", "A", "B", "O", "AB"};
						ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 13, btype[b[0]]);
					}
					break;
				case 0x12:	// astro12
					{
						char* astro[] = {"00", "Ђ", "", "oq", "ɍ", "", "", "V", "", "č", "€", "݂ߍ", "", "ւт", "-"};
						ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 14, astro[b[0]]);
					}
					break;
				case 0x22:	// birthday
					strncopy(buf, b, psize);
					buf[psize] = '\0';
					ListView_SetItemText(GetDlgItem(hWnd, IDC_ADDRLIST), i, 15, buf);
					break;
				}

				b += psize;
			}

			if(inserted)
				i++;
		}
	}
parseError:
	Log("OK\r\n");

	hfree(data);

	return;
error:
	if(endComm(hcom))
		CloseHandle(hcom);
	return;
}

BOOL CALLBACK AddrWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		SendMessage(GetDlgItem(hWnd, IDC_ADDRLIST), LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT, 0xFFFFFFFF);

		{
			LV_COLUMN lvc;

			lvc.mask = LVCF_TEXT | LVCF_WIDTH;

			lvc.cx = 100;
			lvc.pszText = "O";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 0, &lvc);
			lvc.cx = 100;
			lvc.pszText = "";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 1, &lvc);
			lvc.cx = 100;
			lvc.pszText = "dbԍ1";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 2, &lvc);
			lvc.cx = 60;
			lvc.pszText = "dbԍ1";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 3, &lvc);
			lvc.cx = 100;
			lvc.pszText = "E-Mail1";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 4, &lvc);
			lvc.cx = 60;
			lvc.pszText = "E-Mail1";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 5, &lvc);
			lvc.cx = 100;
			lvc.pszText = "";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 6, &lvc);
			lvc.cx = 100;
			lvc.pszText = "O[v";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 7, &lvc);
			lvc.cx = 40;
			lvc.pszText = "No";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 8, &lvc);
			lvc.cx = 100;
			lvc.pszText = "Z";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 9, &lvc);
			lvc.cx = 100;
			lvc.pszText = "GPS";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 10, &lvc);
			lvc.cx = 100;
			lvc.pszText = "ʐ^";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 11, &lvc);
			lvc.cx = 100;
			lvc.pszText = "o^";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 12, &lvc);
			lvc.cx = 40;
			lvc.pszText = "t^";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 13, &lvc);
			lvc.cx = 60;
			lvc.pszText = "";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 14, &lvc);
			lvc.cx = 100;
			lvc.pszText = "a";
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_ADDRLIST), 15, &lvc);
		}
		return TRUE;	// set focus
	case WM_LBUTTONDOWN:
		SendMessage(hWnd, WM_NCLBUTTONDOWN, HTCAPTION, 0);
		break;
	case WM_CLOSE:
		EndDialog(hWnd, 0);
		hAddrWnd = NULL;
		break;
	case WM_COMMAND:
		switch(HIWORD(wParam))
		{
		case 0:
			switch(LOWORD(wParam))
			{
			case IDOK:
				PostMessage(hWnd, WM_CLOSE, 0, 0);
				break;
			case IDC_GETADDR:
				getAddresses(hWnd);
				break;
			}
		}
		break;
	}

	return FALSE;
}
